@@ -132,8 +132,16 @@ get.observed.time.matrix.and.regions <- function(data, loc_data, start.year=1950
 	#cols.endy <- as.integer(substr(names.tfr.data[num.columns], 7,10))
 	cols.endy <- as.integer(substr(names.tfr.data[num.columns], 6,9))
 	start.index <- (1:ncol.tfr)[(cols.starty <= start.year) & (cols.endy > start.year)]
+	if(length(start.index) <= 0) {
+		if(cols.starty[1] > start.year)	start.index <- 1
+		else if(cols.endy[length(cols.endy)] <= start.year) return(NULL)
+	}
 	start.col <- names.tfr.data[num.columns][start.index[1]]
 	present.index <- (1:ncol.tfr)[(cols.endy >= present.year) & (cols.starty <= present.year)]
+	if(length(present.index) <= 0) {
+		if(cols.endy[length(cols.endy)] < present.year) present.index <- length(cols.endy)
+		else if(cols.starty[1] > present.year) return(NULL)
+	}
 	present.col <- names.tfr.data[num.columns][present.index[1]]
 	proj.start.col <- names.tfr.data[num.columns][present.index[1]+1]
 
@@ -175,15 +183,15 @@ get.TFRmatrix.and.regions <- function(tfr_data, ..., verbose=FALSE){
 	ncountries.est <- get.nr.countries.est(meta)
 	replaced.processed <- intersect(tfrs$replaced, country.codes.processed)
 	idx.replaced.processed <- is.element(country.codes.processed, replaced.processed)
-	replaced.processed.est <- country.codes.processed[idx.replaced.processed[1:ncountries.est]]
+	replaced.processed.est <- country.codes.processed[1:ncountries.est][idx.replaced.processed[1:ncountries.est]]
 	
 	added.processed <- intersect(tfrs$added, country.codes.processed)
 	idx.added.processed <- is.element(country.codes.processed, added.processed)
-	added.processed.est <- country.codes.processed[idx.added.processed[1:ncountries.est]]
+	added.processed.est <- country.codes.processed[1:ncountries.est][idx.added.processed[1:ncountries.est]]
 	
 	countries.processed <- intersect(countries, country.codes.processed)
 	idx.countries.processed <- is.element(country.codes.processed, countries.processed)
-	countries.processed.est <- country.codes.processed[idx.countries.processed[1:ncountries.est]]
+	countries.processed.est <- country.codes.processed[1:ncountries.est][idx.countries.processed[1:ncountries.est]]
 
 	if(length(replaced.processed.est)+length(added.processed.est)+length(countries.processed.est)> 0) {
 		cat('\nCountries', paste(c(replaced.processed.est, added.processed.est, 
